@@ -318,7 +318,7 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 				GTK_RESPONSE_HELP,
 				NULL);
 
-	gtk_window_set_title (GTK_WINDOW (dlg), _("Character Codings"));
+	gtk_window_set_title (GTK_WINDOW (dlg), _("Character Encodings"));
 	gtk_window_set_default_size (GTK_WINDOW (dlg), 650, 400);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	
@@ -354,8 +354,9 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 	{
 		gtk_widget_show (error_widget);
 
-		gtk_box_pack_start_defaults (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
-					     error_widget);
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
+		                    error_widget,
+		                    TRUE, TRUE, 0);
 		gtk_container_set_border_width (GTK_CONTAINER (error_widget), 5);			     
 
 		return;
