@@ -44,6 +44,8 @@
 #include <gtk/gtk.h>
 
 #include "gedit-notebook.h"
+#include "gedit-tab.h"
+#include "gedit-tab-label.h"
 #include "gedit-marshal.h"
 #include "gedit-window.h"
 
@@ -66,6 +68,7 @@ struct _GeditNotebookPrivate
 	gint	       always_show_tabs : 1;
 	gint           close_buttons_sensitive : 1;
 	gint           tab_drag_and_drop_enabled : 1;
+	guint          destroy_has_run : 1;
 };
 
 G_DEFINE_TYPE(GeditNotebook, gedit_notebook, GTK_TYPE_NOTEBOOK)
@@ -96,13 +99,39 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+static void
+gedit_notebook_destroy (GtkObject *object)
+{
+	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
+
+	if (!notebook->priv->destroy_has_run)
+	{
+		GList *children, *l;
+
+		children = gtk_container_get_children (GTK_CONTAINER (notebook));
+
+		for (l = children; l != NULL; l = g_list_next (l))
+		{
+			gedit_notebook_remove_tab (notebook,
+						   GEDIT_TAB (l->data));
+		}
+
+		g_list_free (children);
+		notebook->priv->destroy_has_run = TRUE;
+	}
+
+	GTK_OBJECT_CLASS (gedit_notebook_parent_class)->destroy (object);
+}
+
 static void
 gedit_notebook_class_init (GeditNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS (klass);
 	GtkNotebookClass *notebook_class = GTK_NOTEBOOK_CLASS (klass);
 
 	object_class->finalize = gedit_notebook_finalize;
+	gtkobject_class->destroy = gedit_notebook_destroy;
 	
 	notebook_class->change_current_page = gedit_notebook_change_current_page;
 
@@ -157,17 +186,6 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 			      GEDIT_TYPE_TAB);
 
 	g_type_class_add_private (object_class, sizeof(GeditNotebookPrivate));
-	
-	/* Set up a style for the close button with no focus padding. */
-	gtk_rc_parse_string (
-		"style \"gedit-close-button-style\"\n"
-		"{\n"
-		"  GtkWidget::focus-padding = 0\n"
-		"  GtkWidget::focus-line-width = 0\n"
-		"  xthickness = 0\n"
-		"  ythickness = 0\n"
-		"}\n"
-		"widget \"*.gedit-close-button\" style \"gedit-close-button-style\"");
 }
 
 static GeditNotebook *
@@ -722,8 +740,7 @@ gedit_notebook_finalize (GObject *object)
 {
 	GeditNotebook *notebook = GEDIT_NOTEBOOK (object);
 
-	if (notebook->priv->focused_pages)
-		g_list_free (notebook->priv->focused_pages);
+	g_list_free (notebook->priv->focused_pages);
 
 	G_OBJECT_CLASS (gedit_notebook_parent_class)->finalize (object);
 }
@@ -772,195 +789,56 @@ gedit_notebook_change_current_page (GtkNotebook *notebook,
 }
 
 static void
-sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
+close_button_clicked_cb (GeditTabLabel *tab_label, GeditNotebook *notebook)
 {
-	GeditNotebook *nb;
-	GtkWidget *label;
-	GtkWidget *ebox;
-	GtkWidget *button;
-	GtkWidget *spinner;
-	gchar *str;
-	GtkImage *icon;
-	GeditTabState  state;
-	
-	label = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label"));
-	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label-ebox"));
-	icon = GTK_IMAGE (g_object_get_data (G_OBJECT (hbox), "icon"));
-	button = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "close-button"));
-	spinner = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "spinner"));
+	GeditTab *tab;
 
-	nb = GEDIT_NOTEBOOK (gtk_widget_get_parent (GTK_WIDGET (tab)));
-
-	g_return_if_fail ((label   != NULL) &&
-			  (ebox    != NULL) &&
-			  (button  != NULL) &&
-			  (icon    != NULL) &&
-			  (spinner != NULL) &&
-			  (nb      != NULL));
-
-	str = _gedit_tab_get_name (tab);
-	g_return_if_fail (str != NULL);
-	
-	gtk_label_set_text (GTK_LABEL (label), str);
-	g_free (str);
-	
-	str = _gedit_tab_get_tooltips (tab);
-	g_return_if_fail (str != NULL);
-	
-	gtk_widget_set_tooltip_markup (ebox, str);
-	g_free (str);
-		
-	state = gedit_tab_get_state (tab);
-	
-	gtk_widget_set_sensitive (button,
-				  nb->priv->close_buttons_sensitive &&  
-				  (state != GEDIT_TAB_STATE_CLOSING) &&
-				  (state != GEDIT_TAB_STATE_SAVING)  &&
-				  (state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW) &&
-				  (state != GEDIT_TAB_STATE_SAVING_ERROR));
-				  
-	if ((state == GEDIT_TAB_STATE_LOADING)   ||
-	    (state == GEDIT_TAB_STATE_SAVING)    ||
-	    (state == GEDIT_TAB_STATE_REVERTING))
-	{
-		gtk_widget_hide (GTK_WIDGET (icon));
-		
-		gtk_widget_show (spinner);
-#ifdef BUILD_SPINNER
-		gedit_spinner_start (GEDIT_SPINNER (spinner));
-#else
-		gtk_spinner_start (GTK_SPINNER (spinner));
-#endif
-	}
-	else
-	{
-		GdkPixbuf *pixbuf;
-		
-		pixbuf = _gedit_tab_get_icon (tab);
-		gtk_image_set_from_pixbuf (icon, pixbuf);
-
-		if (pixbuf != NULL)
-			g_object_unref (pixbuf);
-
-		gtk_widget_show (GTK_WIDGET (icon));
-		
-		gtk_widget_hide (spinner);
-#ifdef BUILD_SPINNER
-		gedit_spinner_stop (GEDIT_SPINNER (spinner));
-#else
-		gtk_spinner_stop (GTK_SPINNER (spinner));
-#endif
-	}
-}
-
-static void
-close_button_clicked_cb (GtkWidget *widget, 
-			 GtkWidget *tab)
-{
-	GeditNotebook *notebook;
-
-	notebook = GEDIT_NOTEBOOK (gtk_widget_get_parent (tab));
+	tab = gedit_tab_label_get_tab (tab_label);
 	g_signal_emit (notebook, signals[TAB_CLOSE_REQUEST], 0, tab);
 }
 
-static void
-close_button_style_set_cb (GtkWidget *button,
-			   GtkStyle *previous_style,
-			   gpointer user_data)
+static GtkWidget *
+create_tab_label (GeditNotebook *nb,
+		  GeditTab      *tab)
 {
-	gint h, w;
+	GtkWidget *tab_label;
 
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
-					   GTK_ICON_SIZE_MENU, &w, &h);
+	tab_label = gedit_tab_label_new (tab);
 
-	gtk_widget_set_size_request (button, w + 2, h + 2);
+	g_signal_connect (tab_label,
+			  "close-clicked",
+			  G_CALLBACK (close_button_clicked_cb),
+			  nb);
+
+	g_object_set_data (G_OBJECT (tab), "tab-label", tab_label);
+
+	return tab_label;
+}
+
+static void
+remove_tab_label (GeditNotebook *nb,
+		  GeditTab      *tab)
+{
+	GtkWidget *tab_label;
+
+	tab_label = gedit_tab_label_new (tab);
+
+	g_signal_handlers_disconnect_by_func (tab_label,
+					      G_CALLBACK (close_button_clicked_cb),
+					      nb);
+
+	g_object_set_data (G_OBJECT (tab), "tab-label", NULL);
 }
 
 static GtkWidget *
-build_tab_label (GeditNotebook *nb, 
-		 GeditTab      *tab)
+get_tab_label (GeditTab *tab)
 {
-	GtkWidget *hbox, *label_hbox, *label_ebox;
-	GtkWidget *label, *dummy_label;
-	GtkWidget *close_button;
-	GtkWidget *image;
-	GtkWidget *spinner;
-	GtkWidget *icon;
+	GtkWidget *tab_label;
 
-	hbox = gtk_hbox_new (FALSE, 4);
+	tab_label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab), "tab-label"));
+	g_return_val_if_fail (tab_label != NULL, NULL);
 
-	label_ebox = gtk_event_box_new ();
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
-	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
-
-	label_hbox = gtk_hbox_new (FALSE, 4);
-	gtk_container_add (GTK_CONTAINER (label_ebox), label_hbox);
-
-	/* setup close button */
-	close_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (close_button),
-			       GTK_RELIEF_NONE);
-	/* don't allow focus on the close button */
-	gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
-
-	/* make it as small as possible */
-	gtk_widget_set_name (close_button, "gedit-close-button");
-
-	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
-					  GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (close_button), image);
-	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
-
-	gtk_widget_set_tooltip_text (close_button, _("Close document"));
-
-	g_signal_connect (close_button,
-			  "clicked",
-			  G_CALLBACK (close_button_clicked_cb),
-			  tab);
-
-	/* setup spinner */
-#ifdef BUILD_SPINNER
-	spinner = gedit_spinner_new ();
-	gedit_spinner_set_size (GEDIT_SPINNER (spinner), GTK_ICON_SIZE_MENU);
-#else
-	spinner = gtk_spinner_new ();
-#endif
-	gtk_box_pack_start (GTK_BOX (label_hbox), spinner, FALSE, FALSE, 0);
-
-	/* setup site icon, empty by default */
-	icon = gtk_image_new ();
-	gtk_box_pack_start (GTK_BOX (label_hbox), icon, FALSE, FALSE, 0);
-	
-	/* setup label */
-	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start (GTK_BOX (label_hbox), label, FALSE, FALSE, 0);
-
-	dummy_label = gtk_label_new ("");
-	gtk_box_pack_start (GTK_BOX (label_hbox), dummy_label, TRUE, TRUE, 0);
-	
-	/* Set minimal size */
-	g_signal_connect (close_button, "style-set",
-			  G_CALLBACK (close_button_style_set_cb), NULL);
-	
-	gtk_widget_show (hbox);
-	gtk_widget_show (label_ebox);
-	gtk_widget_show (label_hbox);
-	gtk_widget_show (label);
-	gtk_widget_show (dummy_label);	
-	gtk_widget_show (image);
-	gtk_widget_show (close_button);
-	gtk_widget_show (icon);
-	
-	g_object_set_data (G_OBJECT (hbox), "label", label);
-	g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
-	g_object_set_data (G_OBJECT (hbox), "spinner", spinner);
-	g_object_set_data (G_OBJECT (hbox), "icon", icon);
-	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
-	g_object_set_data (G_OBJECT (tab), "close-button", close_button);
-
-	return hbox;
+	return tab_label;
 }
 
 /**
@@ -996,32 +874,17 @@ gedit_notebook_add_tab (GeditNotebook *nb,
 		        gint           position,
 		        gboolean       jump_to)
 {
-	GtkWidget *label;
+	GtkWidget *tab_label;
 
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 
-	label = build_tab_label (nb, tab);
-
-	update_tabs_visibility (nb, TRUE);
-
+	tab_label = create_tab_label (nb, tab);
 	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), 
 				  GTK_WIDGET (tab),
-				  label, 
+				  tab_label,
 				  position);
-
-	sync_name (tab, NULL, label);
-		         
-	g_signal_connect_object (tab, 
-				 "notify::name",
-			         G_CALLBACK (sync_name), 
-			         label, 
-			         0);
-	g_signal_connect_object (tab, 
-				 "notify::state",
-			         G_CALLBACK (sync_name), 
-			         label, 
-			         0);			         
+	update_tabs_visibility (nb, TRUE);
 
 	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, tab);
 
@@ -1076,30 +939,22 @@ static void
 remove_tab (GeditTab      *tab,
 	    GeditNotebook *nb)
 {
-	GtkWidget *label, *ebox;
 	gint position;
 
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 
-	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
-
-	g_signal_handlers_disconnect_by_func (tab,
-					      G_CALLBACK (sync_name), 
-					      label);
-
 	/* we ref the tab so that it's still alive while the tabs_removed
 	 * signal is processed.
 	 */
 	g_object_ref (tab);
 
+	remove_tab_label (nb, tab);
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
-
 	update_tabs_visibility (nb, FALSE);
 
 	g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, tab);
 
-	g_object_unref (tab);	
+	g_object_unref (tab);
 }
 
 /**
@@ -1156,23 +1011,12 @@ static void
 set_close_buttons_sensitivity (GeditTab      *tab,
                                GeditNotebook *nb)
 {
-	GtkWidget     *button;
-	GeditTabState  state;
-	
-	button = GTK_WIDGET (g_object_get_data (G_OBJECT (tab), 
-						"close-button"));	
-	g_return_if_fail (button != NULL);
-	
-	state = gedit_tab_get_state (tab);
-	
-	gtk_widget_set_sensitive (button, 
-				  nb->priv->close_buttons_sensitive &&
-				  (state != GEDIT_TAB_STATE_CLOSING) &&
-				  (state != GEDIT_TAB_STATE_SAVING)  &&
-				  (state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW) &&
-				  (state != GEDIT_TAB_STATE_PRINTING) &&
-				  (state != GEDIT_TAB_STATE_PRINT_PREVIEWING) &&				  
-				  (state != GEDIT_TAB_STATE_SAVING_ERROR));
+	GtkWidget *tab_label;
+
+	tab_label = get_tab_label (tab);
+
+	gedit_tab_label_set_close_button_sensitive (GEDIT_TAB_LABEL (tab_label),
+						    nb->priv->close_buttons_sensitive);
 }
 
 /**
@@ -1187,14 +1031,14 @@ gedit_notebook_set_close_buttons_sensitive (GeditNotebook *nb,
 					    gboolean       sensitive)
 {
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
-	
+
 	sensitive = (sensitive != FALSE);
-	
+
 	if (sensitive == nb->priv->close_buttons_sensitive)
 		return;
-	
+
 	nb->priv->close_buttons_sensitive = sensitive;
-	
+
 	gtk_container_foreach (GTK_CONTAINER (nb),
 			       (GtkCallback)set_close_buttons_sensitivity,
 			       nb);
@@ -1212,7 +1056,7 @@ gboolean
 gedit_notebook_get_close_buttons_sensitive (GeditNotebook *nb)
 {
 	g_return_val_if_fail (GEDIT_IS_NOTEBOOK (nb), TRUE);
-	
+
 	return nb->priv->close_buttons_sensitive;
 }
 
