@@ -25,7 +25,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-statusbar.c 6824 2009-01-07 17:44:15Z pborelli $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -125,10 +125,12 @@ set_statusbar_width_chars (GtkWidget *statusbar,
 	PangoContext *context;
 	PangoFontMetrics *metrics;
 	gint char_width, digit_width, width;
+	GtkStyle *style;
 
 	context = gtk_widget_get_pango_context (statusbar);
+	style = gtk_widget_get_style (GTK_WIDGET (statusbar));
 	metrics = pango_context_get_metrics (context,
-					     GTK_WIDGET (statusbar)->style->font_desc, 
+					     style->font_desc,
 					     pango_context_get_language (context));
 
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
