@@ -121,6 +121,8 @@ void		 gedit_tab_set_auto_save_interval
 						(GeditTab            *tab, 
 						 gint                interval);
 
+void		 gedit_tab_set_info_bar		(GeditTab            *tab,
+						 GtkWidget           *info_bar);
 /*
  * Non exported methods
  */
@@ -144,7 +146,8 @@ void		 _gedit_tab_revert		(GeditTab            *tab);
 void		 _gedit_tab_save		(GeditTab            *tab);
 void		 _gedit_tab_save_as		(GeditTab            *tab,
 						 const gchar         *uri,
-						 const GeditEncoding *encoding);
+						 const GeditEncoding *encoding,
+						 GeditDocumentNewlineType newline_type);
 
 void		 _gedit_tab_print		(GeditTab            *tab);
 void		 _gedit_tab_print_preview	(GeditTab            *tab);
