@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c 6367 2008-08-09 13:20:24Z jessevdk $ 
+ * $Id$ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,10 +43,11 @@
 
 #include "gedit-view.h"
 #include "gedit-debug.h"
+#include "gedit-prefs-manager.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-marshal.h"
 #include "gedit-utils.h"
-#include "sexy-icon-entry.h"
+
 
 #define GEDIT_VIEW_SCROLL_MARGIN 0.02
 #define GEDIT_VIEW_SEARCH_DIALOG_TIMEOUT (30*1000) /* 30 seconds */
@@ -68,9 +69,6 @@ enum
 
 struct _GeditViewPrivate
 {
-	/* idle hack to make open-at-line work */
-	guint        scroll_idle;
-	
 	SearchMode   search_mode;
 	
 	GtkTextIter  start_search_iter;
@@ -110,19 +108,21 @@ static gboolean gedit_view_drag_motion		(GtkWidget        *widget,
 						 GdkDragContext   *context,
 						 gint              x,
 						 gint              y,
-						 guint             time);
+						 guint             timestamp);
 static void     gedit_view_drag_data_received   (GtkWidget        *widget,
 						 GdkDragContext   *context,
 						 gint              x,
 						 gint              y,
 						 GtkSelectionData *selection_data,
 						 guint             info,
-						 guint             time);
+						 guint             timestamp);
 static gboolean gedit_view_drag_drop		(GtkWidget        *widget,
 	      					 GdkDragContext   *context,
 	      					 gint              x,
 	      					 gint              y,
-	      					 guint             time);
+	      					 guint             timestamp);
+static gboolean	gedit_view_button_press_event	(GtkWidget        *widget,
+						 GdkEventButton   *event);
 
 static gboolean start_interactive_search	(GeditView        *view);
 static gboolean start_interactive_goto_line	(GeditView        *view);
@@ -207,7 +207,7 @@ gedit_view_class_init (GeditViewClass *klass)
 	widget_class->drag_motion = gedit_view_drag_motion;
 	widget_class->drag_data_received = gedit_view_drag_data_received;
 	widget_class->drag_drop = gedit_view_drag_drop;
-
+	widget_class->button_press_event = gedit_view_button_press_event;
 	klass->start_interactive_search = start_interactive_search;
 	klass->start_interactive_goto_line = start_interactive_goto_line;
 	klass->reset_searched_text = reset_searched_text;	
@@ -286,15 +286,6 @@ gedit_view_class_init (GeditViewClass *klass)
 				      G_TYPE_INT, 1);
 }
 
-static gboolean
-scroll_to_cursor_on_init (GeditView *view)
-{
-	gedit_view_scroll_to_cursor (view);
-
-	view->priv->scroll_idle = 0;
-	return FALSE;
-}
-
 static void
 current_buffer_removed (GeditView *view)
 {
@@ -381,16 +372,7 @@ gedit_view_init (GeditView *view)
 		      "indent_on_tab", TRUE,
 		      NULL);
 
-	/* Make sure that the view is scrolled to the cursor so
-	 * that "gedit +100 foo.txt" works.
-	 * We would like to this on the first expose handler so that
-	 * it would look instantaneous, but this isn't currently
-	 * possible: see bug #172277 and bug #311728.
-	 * So we need to do this in an idle handler.
-	 */
-	view->priv->scroll_idle = g_idle_add ((GSourceFunc) scroll_to_cursor_on_init, view);
-	
-	view->priv->typeselect_flush_timeout = 0;	
+	view->priv->typeselect_flush_timeout = 0;
 	view->priv->wrap_around = TRUE;
 
 	/* Drag and drop support */	
@@ -413,12 +395,6 @@ gedit_view_destroy (GtkObject *object)
 
 	view = GEDIT_VIEW (object);
 
-	if (view->priv->scroll_idle > 0)
-	{
-		g_source_remove (view->priv->scroll_idle);
-		view->priv->scroll_idle = 0;
-	}
-
 	if (view->priv->search_window != NULL)
 	{
 		gtk_widget_destroy (view->priv->search_window);
@@ -580,6 +556,13 @@ gedit_view_paste_clipboard (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_delete_selection:
+ * @view: a #GeditView
+ * 
+ * Deletes the text currently selected in the #GtkTextBuffer associated
+ * to the view and scroll to the cursor position.
+ **/
 void
 gedit_view_delete_selection (GeditView *view)
 {
@@ -606,6 +589,12 @@ gedit_view_delete_selection (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_select_all:
+ * @view: a #GeditView
+ * 
+ * Selects all the text displayed in the @view.
+ **/
 void
 gedit_view_select_all (GeditView *view)
 {
@@ -623,6 +612,12 @@ gedit_view_select_all (GeditView *view)
 	gtk_text_buffer_select_range (buffer, &start, &end);
 }
 
+/**
+ * gedit_view_scroll_to_cursor:
+ * @view: a #GeditView
+ * 
+ * Scrolls the @view to the cursor position.
+ **/
 void
 gedit_view_scroll_to_cursor (GeditView *view)
 {
@@ -643,6 +638,15 @@ gedit_view_scroll_to_cursor (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_set_font:
+ * @view: a #GeditView
+ * @def: whether to reset the default font
+ * @font_name: the name of the font to use
+ * 
+ * If @def is #TRUE, resets the font of the @view to the default font
+ * otherwise sets it to @font_name.
+ **/
 void
 gedit_view_set_font (GeditView   *view, 
 		     gboolean     def, 
@@ -1253,18 +1257,45 @@ search_entry_insert_text (GtkEditable *editable,
 		gunichar c;
 		const gchar *p;
 	 	const gchar *end;
+	 	const gchar *next;
 
 		p = text;
 		end = text + length;
 
-		while (p != end) {
-			const gchar *next;
+		if (p == end)
+			return;
+
+		c = g_utf8_get_char (p);
+		
+		if (((c == '-' || c == '+') && *position == 0) ||
+		    (c == ':' && *position != 0))
+		{
+			gchar *s = NULL;
+		
+			if (c == ':')
+			{
+				s = gtk_editable_get_chars (editable, 0, -1);
+				s = g_utf8_strchr (s, -1, ':');
+			}
+			
+			if (s == NULL || s == p)
+			{
+				next = g_utf8_next_char (p);
+				p = next;
+			}
+			
+			g_free (s);
+		}
+
+		while (p != end)
+		{
 			next = g_utf8_next_char (p);
 
 			c = g_utf8_get_char (p);
 
 			if (!g_unichar_isdigit (c)) {
 				g_signal_stop_emission_by_name (editable, "insert_text");
+				gtk_widget_error_bell (view->priv->search_entry);
 				break;
 			}
 
@@ -1311,31 +1342,24 @@ search_entry_insert_text (GtkEditable *editable,
 static void
 customize_for_search_mode (GeditView *view)
 {
-	GtkWidget *icon;
-
 	if (view->priv->search_mode == SEARCH)
 	{
-		icon = gtk_image_new_from_stock (GTK_STOCK_FIND,
-						 GTK_ICON_SIZE_MENU);
-
+		gtk_entry_set_icon_from_stock (GTK_ENTRY (view->priv->search_entry),
+					       GTK_ENTRY_ICON_PRIMARY,
+					       GTK_STOCK_FIND);
+		
 		gtk_widget_set_tooltip_text (view->priv->search_entry,
 					     _("String you want to search for"));
 	}
 	else
 	{
-		icon = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO,
-						 GTK_ICON_SIZE_MENU);
-
+		gtk_entry_set_icon_from_stock (GTK_ENTRY (view->priv->search_entry),
+					       GTK_ENTRY_ICON_PRIMARY,
+					       GTK_STOCK_JUMP_TO);
+		
 		gtk_widget_set_tooltip_text (view->priv->search_entry,
 					     _("Line you want to move the cursor to"));
 	}
-
-	gtk_widget_show (icon);
-	sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(view->priv->search_entry),
-				  SEXY_ICON_ENTRY_PRIMARY,
-				  GTK_IMAGE (icon));
-
-	g_object_unref (icon);
 }
 
 static gboolean
@@ -1453,7 +1477,7 @@ ensure_search_window (GeditView *view)
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
 
 	/* add entry */
-	view->priv->search_entry = sexy_icon_entry_new ();				  
+	view->priv->search_entry = gtk_entry_new ();
 	gtk_widget_show (view->priv->search_entry);
 	
 	g_signal_connect (view->priv->search_entry, "populate_popup",
@@ -1649,14 +1673,61 @@ search_init (GtkWidget *entry,
 	{
 		if (*entry_text != '\0')
 		{
-			gboolean moved;
+			gboolean moved, moved_offset;
 			gint line;
-
-			line = MAX (atoi (entry_text) - 1, 0);
+			gint offset_line = 0;
+			gint line_offset = 0;
+			gchar **split_text = NULL;
+			const gchar *text;
+			
+			split_text = g_strsplit (entry_text, ":", -1);
+			
+			if (g_strv_length (split_text) > 1)
+			{
+				text = split_text[0];
+			}
+			else
+			{
+				text = entry_text;
+			}
+			
+			if (*text == '-')
+			{
+				gint cur_line = gtk_text_iter_get_line (&view->priv->start_search_iter);
+			
+				if (*(text + 1) != '\0')
+					offset_line = MAX (atoi (text + 1), 0);
+				
+				line = MAX (cur_line - offset_line, 0);
+			}
+			else if (*entry_text == '+')
+			{
+				gint cur_line = gtk_text_iter_get_line (&view->priv->start_search_iter);
+			
+				if (*(text + 1) != '\0')
+					offset_line = MAX (atoi (text + 1), 0);
+				
+				line = cur_line + offset_line;
+			}
+			else
+			{
+				line = MAX (atoi (text) - 1, 0);
+			}
+			
+			if (split_text[1] != NULL)
+			{
+				line_offset = atoi (split_text[1]);
+			}
+			
+			g_strfreev (split_text);
+			
 			moved = gedit_document_goto_line (doc, line);
+			moved_offset = gedit_document_goto_line_offset (doc, line,
+									line_offset);
+			
 			gedit_view_scroll_to_cursor (view);
 
-			if (!moved)
+			if (!moved || !moved_offset)
 				set_entry_background (view->priv->search_entry,
 						      GEDIT_SEARCH_ENTRY_NOT_FOUND);
 			else
@@ -1799,19 +1870,19 @@ gedit_view_drag_motion (GtkWidget      *widget,
 			GdkDragContext *context,
 			gint            x,
 			gint            y,
-			guint           time)
+			guint           timestamp)
 {
 	gboolean result;
 
 	/* Chain up to allow textview to scroll and position dnd mark, note 
 	 * that this needs to be checked if gtksourceview or gtktextview
 	 * changes drag_motion behaviour */
-	result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_motion (widget, context, x, y, time);
+	result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_motion (widget, context, x, y, timestamp);
 
 	/* If this is a URL, deal with it here */
 	if (drag_get_uri_target (widget, context) != GDK_NONE) 
 	{
-		gdk_drag_status (context, context->suggested_action, time);
+		gdk_drag_status (context, context->suggested_action, timestamp);
 		result = TRUE;
 	}
 
@@ -1825,7 +1896,7 @@ gedit_view_drag_data_received (GtkWidget        *widget,
 			       gint              y,
 			       GtkSelectionData *selection_data,
 			       guint             info,
-			       guint             time)
+			       guint             timestamp)
 {
 	gchar **uri_list;
 	
@@ -1839,12 +1910,12 @@ gedit_view_drag_data_received (GtkWidget        *widget,
 			g_signal_emit (widget, view_signals[DROP_URIS], 0, uri_list);
 			g_strfreev (uri_list);
 			
-			gtk_drag_finish (context, TRUE, FALSE, time);
+			gtk_drag_finish (context, TRUE, FALSE, timestamp);
 		}
 	}
 	else
 	{
-		GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_data_received (widget, context, x, y, selection_data, info, time);
+		GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_data_received (widget, context, x, y, selection_data, info, timestamp);
 	}
 }
 
@@ -1853,7 +1924,7 @@ gedit_view_drag_drop (GtkWidget      *widget,
 		      GdkDragContext *context,
 		      gint            x,
 		      gint            y,
-		      guint           time)
+		      guint           timestamp)
 {
 	gboolean result;
 	GdkAtom target;
@@ -1863,18 +1934,82 @@ gedit_view_drag_drop (GtkWidget      *widget,
 
 	if (target != GDK_NONE)
 	{
-		gtk_drag_get_data (widget, context, target, time);
+		gtk_drag_get_data (widget, context, target, timestamp);
 		result = TRUE;
 	}
 	else
 	{
 		/* Chain up */
-		result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_drop (widget, context, x, y, time);
+		result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_drop (widget, context, x, y, timestamp);
 	}
 
 	return result;
 }
 
+static void
+show_line_numbers_toggled (GtkMenu   *menu,
+			   GeditView *view)
+{
+	gboolean show;
+
+	show = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu));
+
+	gedit_prefs_manager_set_display_line_numbers (show);
+}
+
+static GtkWidget *
+create_line_numbers_menu (GtkWidget *view)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+
+	menu = gtk_menu_new ();
+
+	item = gtk_check_menu_item_new_with_mnemonic (_("_Display line numbers"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+					gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (view)));
+	g_signal_connect (item, "toggled",
+			  G_CALLBACK (show_line_numbers_toggled), view);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	
+	gtk_widget_show_all (menu);
+	
+	return menu;
+}
+
+static void
+show_line_numbers_menu (GtkWidget      *view,
+			GdkEventButton *event)
+{
+	GtkWidget *menu;
+
+	menu = create_line_numbers_menu (view);
+
+	gtk_menu_popup (GTK_MENU (menu), 
+			NULL, 
+			NULL,
+			NULL, 
+			NULL,
+			event->button, 
+			event->time);
+}
+
+static gboolean
+gedit_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
+{
+	if ((event->type == GDK_BUTTON_PRESS) && 
+	    (event->button == 3) &&
+	    (event->window == gtk_text_view_get_window (GTK_TEXT_VIEW (widget),
+						        GTK_TEXT_WINDOW_LEFT)))
+	{
+		show_line_numbers_menu (widget, event);
+
+		return TRUE;
+	}
+
+	return GTK_WIDGET_CLASS (gedit_view_parent_class)->button_press_event (widget, event);
+}
+
 static void 	
 search_highlight_updated_cb (GeditDocument *doc,
 			     GtkTextIter   *start,
